#include "DXBuffer.h"
#include <iostream>
#ifdef WIN32
#include "Render3D.h"
#include <d3d11.h>
#endif

using namespace Framework;

// Inhalt der DXBuffer Klasse

// Konstruktor
//  bind: Der verwendungszweck des Buffers. Beispiel: D3D11_BIND_INDEX_BUFFER, D3D11_BIND_VERTEX_BUFFER.
//  eLn: Lnge eines einzelnen Elements in Bytes
DXBuffer::DXBuffer( int eLen )
{
#ifdef WIN32
    buffer = 0;
    description = new D3D11_BUFFER_DESC();
    memset( description, 0, sizeof( description ) );
    description->Usage = D3D11_USAGE_DYNAMIC;
    description->CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
#endif
    data = 0;
    changed = 0;
    altLen = 0;
    elLem = eLen;
    ref = 1;
}

// Destruktor
DXBuffer::~DXBuffer()
{
#ifdef WIN32
    if( buffer )
        buffer->Release();
    delete description;
#endif
}

// Setzt den gendert flg, so das beim nchsten auruf von 'kopieren' die daten neu kopiert werden
void DXBuffer::setChanged()
{
    changed = 1;
}

// ndert die lnge des Buffers beim nchsten aufruf von 'kopieren'
//  ln: Die Lnge in Bytes
void DXBuffer::setLength( int len )
{
    description->ByteWidth = len;
    changed = 1;
}

// Legt fest, was beim nchsten aufruf von 'kopieren' kopiert wird
//  data: Ein zeiger auf die Daten
void DXBuffer::setData( void *data )
{
    this->data = data;
    changed = 1;
}

// Kopiert die Daten in den Buffer, fals sie sich verndert haben
//  zRObj: Das Objekt, mit dem die Grafikkarte angesprochen wird
void DXBuffer::copieren( Render3D *zRObj )
{
#ifdef WIN32
    if( !changed || !description->ByteWidth || !data )
        return;
    if( description->ByteWidth != altLen )
    {
        if( buffer )
            buffer->Release();
        buffer = 0;
    }
    if( !buffer )
    {
        D3D11_SUBRESOURCE_DATA ini;
        memset( &ini, 0, sizeof( ini ) );
        ini.pSysMem = data;

        zRObj->zDevice()->CreateBuffer( description, &ini, &buffer );
        altLen = description->ByteWidth;
    }
    else if( changed )
    {
        D3D11_MAPPED_SUBRESOURCE map;
        zRObj->zContext()->Map( buffer, 0, D3D11_MAP::D3D11_MAP_WRITE_DISCARD, 0, &map );
        memcpy( map.pData, data, description->ByteWidth );
        zRObj->zContext()->Unmap( buffer, 0 );
        changed = 0;
    }
#endif
}

// Gibt die Lnge eines Elementes in bytes zurck
int DXBuffer::getElementLength() const
{
    return elLem;
}
#ifdef WIN32
// Gibt den Buffer zurck
ID3D11Buffer *DXBuffer::zBuffer() const
{
    return buffer;
}
#endif
// Gibt die Anzahl der Elemente im Buffer zurck
int DXBuffer::getElementAnzahl() const
{
    return altLen / elLem;
}

// Erhht den Reference Counting Zhler.
//  return: this.
DXBuffer *DXBuffer::getThis()
{
    ref++;
    return this;
}

// Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
//  return: 0.
DXBuffer *DXBuffer::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}


// Inhalt der DXVertexBuffer Klasse

// Konstruktor
// eSize: Die Lnge eines Elementes in Bytes
DXVertexBuffer::DXVertexBuffer( int eSize )
    : DXBuffer( eSize )
{
#ifdef WIN32
    description->BindFlags = D3D11_BIND_VERTEX_BUFFER;
#endif
}

// Destruktor
DXVertexBuffer::~DXVertexBuffer()
{}


// Inhalt der DXIndexBuffer Klasse

// Konstruktor
// eSize: Die Lnge eines Elementes in Bytes
DXIndexBuffer::DXIndexBuffer( int eSize )
    : DXBuffer( eSize )
{
#ifdef WIN32
    description->BindFlags = D3D11_BIND_INDEX_BUFFER;
#endif
}

// Destruktor
DXIndexBuffer::~DXIndexBuffer()
{}


#ifdef WIN32
// Inhalt der DXStructuredBuffer Klasse

// Konstruktor
// eSize: Die Lnge eines Elementes in Bytes
DXStructuredBuffer::DXStructuredBuffer( int eSize )
    : DXBuffer( eSize )
{
    description->BindFlags = D3D11_BIND_UNORDERED_ACCESS | D3D11_BIND_SHADER_RESOURCE;
    description->MiscFlags = D3D11_RESOURCE_MISC_BUFFER_STRUCTURED;
    description->StructureByteStride = eSize;
    description->Usage = D3D11_USAGE_DEFAULT;
    view = 0;
}

// Destruktor
DXStructuredBuffer::~DXStructuredBuffer()
{
    if( view )
        view->Release();
}

// Kopiert die Daten in den Buffer, fals sie sich verndert haben
//  zRObj: Das Objekt, mit dem die Grafikkarte angesprochen wird
void DXStructuredBuffer::copieren( Render3D *zRObj )
{
    ID3D11Buffer *old = buffer;
    DXBuffer::copieren( zRObj );
    if( buffer != old )
    {
        if( view )
            view->Release();
        D3D11_SHADER_RESOURCE_VIEW_DESC desc = {};
        desc.ViewDimension = D3D11_SRV_DIMENSION_BUFFEREX;
        desc.BufferEx.FirstElement = 0;
        desc.Format = DXGI_FORMAT_UNKNOWN;
        desc.BufferEx.NumElements = description->ByteWidth / description->StructureByteStride;
        zRObj->zDevice()->CreateShaderResourceView( buffer, &desc, &view );
    }
}

// Gibt die verwendtete Shader Resource View zurck
DXStructuredBuffer::operator ID3D11ShaderResourceView *( ) const
{
    return view;
}
#endif